/*
 * Copyright (c) 2008 Jiri Svoboda
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup loader
 * @brief	Loads and runs programs from VFS.
 * @{
 */ 
/**
 * @file
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <as.h>

#include <elf.h>
#include <elf_load.h>
#include <pcb.h>

#define RTLD_BIAS 0x80000

int main(int argc, char *argv[])
{
	elf_info_t prog_info;
	elf_info_t interp_info;
	char *file_name;
	pcb_t *pcb;
	int rc;

	printf("This is loader\n");
	getchar();

	printf("Load program\n");

	rc = elf_load_file("/dltest", 0, &prog_info);
//	rc = elf_load_file("/tetris", 0, &prog_info);
	if (rc < 0) {
		printf("failed to load program\n");
		return 1;
	}

	printf("Create PCB\n");
	if (elf_create_pcb(&prog_info) < 0) return 1;

//	elf_run(&prog_info);

//	getchar();

	printf("Load dynamic linker\n");
	file_name = "/rtld.so";
	printf("open and read '%s'...\n", file_name);
	rc = elf_load_file(file_name, RTLD_BIAS, &interp_info);
	if (rc < 0) {
		printf("failed to load dynamic linker\n");
		return 1;
	}

	/*
	 * Provide rtld with some useful data
	 */
	pcb = (pcb_t *)PCB_ADDRESS;
	pcb->rtld_dynamic = interp_info.dynamic;
	pcb->rtld_bias = RTLD_BIAS;

	printf("run dynamic linker\n");
	elf_run(&interp_info);

	/* not reached */
	return 0;
}

/** @}
 */
